(function($) {

  EdlpCorpus = function(){
    var $container = $('body');
    var $canvas = $('<canvas>').addClass('edlp-map').appendTo($container);
    var canvas = $canvas[0];
    var ctx = canvas.getContext('2d');
    var physics = new Physics();
    var particules = [];

    function init(){
      console.log("EdlpCorpus init()");
      initMap();
    };

    function initMap(){
      console.log("EdlpCorpus initMap()");
      // console.log('physics',physics);
      initCanvas();
      loadCorpus();
    };

    function initCanvas(){
      // resize the canvas to fill browser window dynamically
      window.addEventListener('resize', resizeCanvas, false);
      resizeCanvas();
    };

    function resizeCanvas() {
      canvas.width = window.innerWidth;
      canvas.height = window.innerHeight;
    };

    function loadCorpus(){
      // console.log('drupalSettings',drupalSettings);
      console.log('window.location', window.location);
      var path = window.location.origin + drupalSettings.basepath + "edlp/corpus";
      $.getJSON(path, {}, function(data){
        console.log('corpus loaded : data', data);
        // console.log('first node', data.nodes[0]);
        buildMap(data.nodes);
        startAnime();
      });
    };

    function buildMap(nodes){
      var x,y,p;
      var mass = 8;
      console.log("physics", physics);
      for (var i in nodes) {
        // console.log(nodes[i]);
        x = Math.random()*canvas.width;
        y = Math.random()*canvas.height;
        p = physics.makeParticle(mass, x, y);
        p.velocity = new Physics.Vector((Math.random()-0.5)*0.10, (Math.random()-0.5)*0.10);
        // if(particules.length){
          // for (var i = 0; i < particules.length; i++) {
          //   physics.makeAttraction(p, particules[i], strength, minDistance);
          // }
        // }
        particules.push(p);
      }
    };

    function render(){
      ctx.clearRect(0, 0, canvas.width, canvas.height);
      var radius = 5;

      // colisions between particules
      for (var p = 0, l = particules.length; p < l; p++) {
        var particule = particules[p];
        // var theta = 360 / (Math.PI * radius);
        if(!particule.attracted) continue;
        for (var q = 0; q < p; q++) {
          if(q==p) continue;

          var a = particule;
          var b = particules[q];
          if(!b.attracted) continue;
          var d = a.distanceTo(b);

          // apply new forces if colliding
          if(d <= radius*2){

            var newVelX1 = (a.velocity.x * (a.mass - b.mass) + (2 * b.mass * b.velocity.x)) / (a.mass + b.mass);
            var newVelY1 = (a.velocity.y * (a.mass - b.mass) + (2 * b.mass * b.velocity.y)) / (a.mass + b.mass);
            var newVelX2 = (b.velocity.x * (b.mass - a.mass) + (2 * a.mass * a.velocity.x)) / (a.mass + b.mass);
            var newVelY2 = (b.velocity.y * (b.mass - a.mass) + (2 * a.mass * a.velocity.y)) / (a.mass + b.mass);

            a.velocity.x = newVelX1;
            a.velocity.y = newVelY1;
            b.velocity.x = newVelX2;
            b.velocity.y = newVelY2;

            a.velocity.multiplyScalar(0.75);
            b.velocity.multiplyScalar(0.75);


            // move particles if they overlap
            if (d < radius*2) {
              var makeup = radius - d/2;
              var angle = Math.atan2(b.position.y - a.position.y, b.position.x - a.position.x);

              b.position.x += makeup * Math.cos(angle);
              b.position.y += makeup * Math.sin(angle);

              angle += Math.PI;

              a.position.x += makeup * Math.cos(angle);
              a.position.y += makeup * Math.sin(angle);

            }
          }
        }
      }

      for (var i = 0; i < particules.length; i++) {
        var p = particules[i];
        // wall boucing
        if(p.position.x < 0+radius || p.position.x > canvas.width-radius){
          // console.log("bounce x");
          p.velocity.multiplyScalarXY(-1, 1).multiplyScalar(0.75);
        }
        if(p.position.y < 0+radius || p.position.y > canvas.height-radius){
          // console.log("bounce y");
          p.velocity.multiplyScalarXY(1,-1).multiplyScalar(0.75);
        }



        // draw the ball
        ctx.beginPath();
        ctx.fillRect(p.position.x, p.position.y, 8, 8);
        if(p.attracted){
          ctx.fillStyle = "red";
        }else{
          ctx.fillStyle = "black";
        }
        ctx.fill();
      }


    };

    function startAnime(){
      physics.onUpdate(render);
      // render();

      physics.play()
    };



    init();
  }


  $(document).ready(function($) {
    var edlpcorpus = new EdlpCorpus();
  });


})(jQuery);
